#    This file is part of the LinuxCNC Machine Builder, a graphical
#    configuration editor for LinuxCNC. Copyright 2013 Arvid Brodin.
#
#    Although all the code is new, this program is influenced by
#    PNCconf (Copyright 2009 Chris Morley), in turn based on stepconf,
#    a graphical configuration editor for linuxcnc (Copyright 2007 Jeff
#    Epler).
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from net import Net

PIN_OUT, PIN_IN, PIN_BIDIR = range(3)

### Base class for HAL components
class HALComp(object):
	def __init__(self):
		# Dictionary of net, pinname
		#   This gives us both a list of all the pins of this
		# HAL component, and also which net it is useful to
		# connect each pin to.
		#   Each HAL component must fill this in in its
		# constructor.
		self._pins = {}

	def generate_params(self, file):
		# Write previously generated parameters to file.
		for param in self._params:
			print >>file, ("setp   " + (self.get_halname() + "." + param[0]).ljust(45) + " " + param[1])
		return len(self._params)

	# Child classes may override this:
	def setup_params(self, jointno):
		#   Create a list of the HAL component's parameters
		# (i.e. pins that are set using setp). We cannot do
		# this in the constructor, since many components use
		# references to the INI file, and thus needs to know
		# its joint number.
		#   Each HAL component that has any parameters must
		# implement its own version of this method.
		self._params = []

	# Child classes MUST override this:
	def get_halname(self):
		# Each HAL component must implement this function. It
		# should return the fully qualified HAL name of the
		# component, but exclude specific pin names. Some
		# examples:
		#   Encoder:	"hm2_5i25.0.encoder.00"
		#   Input:	"hm2_5i25.0.7i77.0.0.input-02"
		raise Exception("Abstract method called")

	def get_pinname(self, netname):
		# Get the fully qualified HAL pin name that connects
		# to the net 'netname'. Generic inputs and outputs
		# can connect to _any_ net, so they just return their
		# get_halname().
		if not netname in self._pins:
			return self.get_halname()
		return self.get_halname() + "." + self._pins[netname][0]

	# Child classes may override this:
	def get_pindir(self, netname):
		# Get the direction of the pin that correspond to the
		# net 'netname'.
		return self._pins[netname][1]


### LinuxCNC core
class Pid(HALComp):
	def __init__(self, jointno):
		HALComp.__init__(self)
		self._jointno = jointno
		# Map net names to pin names
		self._pins["pos-cmd"] = ["command", PIN_IN]
		self._pins["vel-fb"] = ["command-deriv", PIN_IN]
		self._pins["pos-fb"] = ["feedback", PIN_IN]
		self._pins["output"] = ["output", PIN_OUT]
		self._pins["enable"] = ["enable", PIN_IN]
		self._pins["index-enable"] = ["index-enable", PIN_IN]
		"""
		Unused pins:
		self._pins[""] = ["feedback-deriv", PIN_IN]
		self._pins[""] = ["error-previous-target", PIN_IN]
		self._pins[""] = ["error", PIN_OUT]
		self._pins[""] = ["saturated", PIN_OUT]
		self._pins[""] = ["saturated-s", PIN_OUT]
		self._pins[""] = ["saturated-count", PIN_OUT]
		"""

	def setup_params(self, jointno):
		self._params = []
		self._params.append(["Pgain", "[AXIS_%d]P" % jointno])
		self._params.append(["Igain", "[AXIS_%d]I" % jointno])
		self._params.append(["Dgain", "[AXIS_%d]D" % jointno])
		self._params.append(["bias", "[AXIS_%d]BIAS" % jointno])
		self._params.append(["FF0", "[AXIS_%d]FF0" % jointno])
		self._params.append(["FF1", "[AXIS_%d]FF1" % jointno])
		self._params.append(["FF2", "[AXIS_%d]FF2" % jointno])
		self._params.append(["deadband", "[AXIS_%d]DEADBAND" % jointno])
		self._params.append(["maxoutput", "[AXIS_%d]MAX_OUTPUT" % jointno])
		"""
		Unused parameters:
		self._params.append(["maxerror", "?"])
		self._params.append(["maxerrorD", "?"])
		self._params.append(["maxerrorI", "?"])
		self._params.append(["maxcmdD", "?"])
		self._params.append(["maxcmdDD", "?"])
		"""

	def get_halname(self):
		return "pid.%d" % self._jointno


class Joint(HALComp):
	def __init__(self, jointno):
		HALComp.__init__(self)
		self._jointno = jointno
		self._pins["enable"] = ["amp-enable-out", PIN_OUT]
		self._pins["home"] = ["home-sw-in", PIN_IN]
		self._pins["index-enable"] = ["index_enable", PIN_BIDIR]
		self._pins["pos-cmd"] = ["motor-pos-cmd", PIN_OUT]
		self._pins["pos-fb"] = ["motor-pos-fb", PIN_IN]
		self._pins["min-lim"] = ["neg-lim-sw-in", PIN_IN]
		self._pins["max-lim"] = ["pos-lim-sw-in", PIN_IN]
		"""
		Unused pins:
		self._pins[""] = ["amp-fault-in", PIN_IN]
		self._pins[""] = ["homing", PIN_OUT]
		self._pins[""] = ["is-unlocked", PIN_IN]
		self._pins[""] = ["jog-counts", PIN_IN]
		self._pins[""] = ["jog-enable", PIN_IN]
		self._pins[""] = ["jog-scale", PIN_IN]
		self._pins[""] = ["jog-vel-mode", PIN_IN]
		self._pins[""] = ["joint-pos-cmd", PIN_OUT]
		self._pins[""] = ["joint-pos-fb", PIN_OUT]
		self._pins[""] = ["unlock", PIN_OUT]
		"""

	def get_halname(self):
		return "axis.%d" % self._jointno
